!--------------------------------------------------------------------
subroutine caluu1(uri,uu,x1,is1,ie1,js1,je1,ks1,ke1)
!--------------------------------------------------------------------
!
! Calculation of conserved variables (uu) from primitive variables (uri)
!
  use pram, only : imax, jmax, kmax, nv, ns, gam, c0, ieos, iflag  
  implicit none

  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1
       
  real(8) :: uri(ns,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax)      

  real(8) :: de, v1, v2, v3, pr, b1, b2, b3, e1, e2, e3, ch, ph, ps
  real(8) :: roh, roe, vsq, bsq, esq, gfl, denr, eem
!      
!=====================================================================

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1

        de=uri(1,i,j,k)  ! density
        v1=uri(2,i,j,k)  ! velocity_1
        v2=uri(3,i,j,k)  ! velocity_2
        v3=uri(4,i,j,k)  ! velocity_3
        pr=uri(5,i,j,k)  ! gas pressure
        b1=uri(7,i,j,k)  ! B-field_1
        b2=uri(8,i,j,k)  ! B-field_2
        b3=uri(9,i,j,k)  ! B-field_3
        e1=uri(10,i,j,k) ! E-field_1
        e2=uri(11,i,j,k) ! E-field_2
        e3=uri(12,i,j,k) ! E-field_3
        ch=uri(13,i,j,k) ! charge density
        ph=uri(14,i,j,k) ! Phi for GLM
        ps=uri(15,i,j,k) ! Psi for GLM  

        if(ieos .eq. 0) then
          roh=de+(gam/(gam-1.0))*pr
        elseif(ieos .eq. 1) then
          roh=(5./2.)*pr+sqrt((9./4.)*pr**2+de**2)
        elseif(ieos .eq. 2) then
          roe=(3./2.)*(pr+((3.*pr**2)/(2.0*de+sqrt(2.*pr**2+4.*de**2)) ))
          roh=de+roe+pr
        elseif(ieos .eq. 3) then
          roh=(12.*pr**2+8.*pr*de+2.*de**2)/(3.*pr+2.*de)
        endif
         
        vsq=v1*v1+v2*v2+v3*v3
        bsq=(b1*b1+b2*b2+b3*b3)
        esq=(e1*e1+e2*e2+e3*e3)
        gfl=1.0/sqrt(1.0-vsq)
 
        denr=roh*(gfl**2)
        eem=0.5*(esq+bsq)
          
        uu(1,i,j,k)=gfl*de                ! \gamma*density
        uu(2,i,j,k)=denr*v1+(e2*b3-b2*e3) ! Momentum_1
        uu(3,i,j,k)=denr*v2+(e3*b1-b3*e1) ! Momentum_2
        uu(4,i,j,k)=denr*v3+(e1*b2-b1*e2) ! Momentum_3
        uu(5,i,j,k)=denr-pr+eem           ! total energy
           
        if( iflag(6).le.1 ) then
          uu(6,i,j,k)=gfl*de*x1(i)
        elseif( iflag(6).le.3 ) then
          uu(6,i,j,k)=gfl*pr/de**(gam-1.0)
        elseif( iflag(6).eq.4 ) then
          uu(6,i,j,k)=gfl*de*log(pr/de**gam)
        endif         
           
        uu(7,i,j,k)=b1
        uu(8,i,j,k)=b2
        uu(9,i,j,k)=b3
        uu(10,i,j,k)=e1
        uu(11,i,j,k)=e2
        uu(12,i,j,k)=e3
        uu(13,i,j,k)=ch
!
        uu(14,i,j,k)=ph
        uu(15,i,j,k)=ps
!
      enddo
    enddo
  enddo
      
  return
end subroutine caluu1
!
!--------------------------------------------------------------------@
subroutine caluu2(uri,uu,x1,is1,ie1,js1,je1,ks1,ke1)
!--------------------------------------------------------------------@
!
! Calculation of conserved variables (uu) from primitive variables (uri)
!
  use pram, only : imax, jmax, kmax, nv, ns, gam, c0, ieos, iflag  
  implicit none

  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1
       
  real(8) :: uri(ns,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax)      

  real(8) :: de, v1, v2, v3, pr, b1, b2, b3, e1, e2, e3, ch, ph, ps, sg 
  real(8) :: roh, roe, vsq, bsq, esq, gfl, denr, eem
      
!=====================================================================

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1

        de=uri(1,i,j,k)
        v1=uri(2,i,j,k)
        v2=uri(3,i,j,k)
        v3=uri(4,i,j,k)
        pr=uri(5,i,j,k)
        b1=uri(7,i,j,k)
        b2=uri(8,i,j,k)
        b3=uri(9,i,j,k)
        e1=uri(10,i,j,k)
        e2=uri(11,i,j,k)
        e3=uri(12,i,j,k)
        ch=uri(13,i,j,k)
        ph=uri(14,i,j,k)
        ps=uri(15,i,j,k)
         
        if(ieos .eq. 0) then
          roh=de+(gam/(gam-1.0))*pr
        elseif(ieos .eq. 1) then
          roh=(5./2.)*pr+sqrt((9./4.)*pr**2+de**2)
        elseif(ieos .eq. 2) then
          roe=(3./2.)*(pr+((3.*pr**2)/(2.0*de+sqrt(2.*pr**2+4.*de**2)) ))
          roh=de+roe+pr
        elseif(ieos .eq. 3) then
          roh=(12.*pr**2+8.*pr*de+2.*de**2)/(3.*pr+2.*de)
        endif
          
        vsq=v1*v1+v2*v2+v3*v3
        bsq=(b1*b1+b2*b2+b3*b3)
        esq=(e1*e1+e2*e2+e3*e3)
        gfl=1.0/sqrt(1.0-vsq)
!
        denr=(gfl**2)*roh
        eem=0.5*(bsq+esq)
          
        uu(1,i,j,k)=gfl*de
        uu(2,i,j,k)=denr*v1+(e2*b3-b2*e3)
        uu(3,i,j,k)=denr*v2+(e3*b1-b3*e1)
        uu(4,i,j,k)=denr*v3+(e1*b2-b1*e2)
        uu(5,i,j,k)=denr-pr+eem
           
        if( iflag(6).le.1 ) then
          uu(6,i,j,k)=gfl*de*x1(i)
        elseif( iflag(6).le.3 ) then
          uu(6,i,j,k)=gfl*pr/de**(gam-1.0)
        elseif( iflag(6).eq.4 ) then
          uu(6,i,j,k)=gfl*de*log(pr/de**gam)
        endif         
           
        uu(7,i,j,k)=b1
        uu(8,i,j,k)=b2
        uu(9,i,j,k)=b3
        uu(10,i,j,k)=e1
        uu(11,i,j,k)=e2
        uu(12,i,j,k)=e3
        uu(13,i,j,k)=ch
!
        uu(14,i,j,k)=ph
        uu(15,i,j,k)=ps
!
      enddo
    enddo
  enddo
      
   return
end subroutine caluu2
!
!--------------------------------------------------------------------@
subroutine caluu2a(uri,uu,is1,ie1,js1,je1,ks1,ke1)
!--------------------------------------------------------------------@
!
! Calculation of conserved variables (uu) from primitive variables (uri)
!
  use pram, only : imax, jmax, kmax, nv, ns, metric, gam, c0, ieos
  implicit none

  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1
       
  real(8) :: uri(ns,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: de, v1, v2, v3, pr, b1, b2, b3, e1, e2, e3, ch, ps, ph 
  real(8) :: roh, roe, vsq, bsq, esq, gfl, denr, eem
      
!=====================================================================

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1

        de=uri(1,i,j,k)
        v1=uri(2,i,j,k)
        v2=uri(3,i,j,k)
        v3=uri(4,i,j,k)
        pr=uri(5,i,j,k)
        b1=uri(7,i,j,k)
        b2=uri(8,i,j,k)
        b3=uri(9,i,j,k)
        e1=uri(10,i,j,k)
        e2=uri(11,i,j,k)
        e3=uri(12,i,j,k)
        ch=uri(13,i,j,k)
        ph=uri(14,i,j,k)
        ps=uri(15,i,j,k)
         
        if(ieos .eq. 0) then
          roh=de+(gam/(gam-1.0))*pr
        elseif(ieos .eq. 1) then
          roh=(5./2.)*pr+sqrt((9./4.)*pr**2+de**2)
        elseif(ieos .eq. 2) then
          roe=(3./2.)*(pr+((3.*pr**2)/(2.0*de+sqrt(2.*pr**2+4.*de**2)) ))
          roh=de+roe+pr
        elseif(ieos .eq. 3) then
          roh=(12.*pr**2+8.*pr*de+2.*de**2)/(3.*pr+2.*de)
        endif
          
        vsq=v1*v1+v2*v2+v3*v3
        bsq=(b1*b1+b2*b2+b3*b3)
        esq=(e1*e1+e2*e2+e3*e3)
        gfl=1.0/sqrt(1.0-vsq)

        denr=(gfl**2)*roh
        eem=0.5*(bsq+esq)
          
        uu(1,i,j,k)=gfl*de
        uu(2,i,j,k)=denr*v1+(e2*b3-b2*e3)
        uu(3,i,j,k)=denr*v2+(e3*b1-b3*e1)
        uu(4,i,j,k)=denr*v3+(e1*b2-b1*e2)
        uu(5,i,j,k)=denr-pr+eem
           
        uu(7,i,j,k)=b1
        uu(8,i,j,k)=b2
        uu(9,i,j,k)=b3
        uu(10,i,j,k)=e1
        uu(11,i,j,k)=e2
        uu(12,i,j,k)=e3
        uu(13,i,j,k)=ch
!
        uu(14,i,j,k)=ph
        uu(15,i,j,k)=ps
!
      enddo
    enddo
  enddo
      
  return
end subroutine caluu2a
!
!--------------------------------------------------------------------
subroutine caluu3(uri,uu,x1,nm0,is1,ie1,js1,je1,ks1,ke1)
!--------------------------------------------------------------------
!
! Calculation of conserved variables (uu) from primitive variables (uri)
!
  use pram, only : imax, jmax, kmax, nv, ns, gam, c0, ieos, iflag  
  implicit none

  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1
  integer :: nm0
       
  real(8) :: uri(ns,is1:ie1,js1:je1,ks1:ke1)
  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: x1(imax)

  real(8) :: de, v1, v2, v3, pr, b1, b2, b3, e1, e2, e3, ch, ph, ps 
  real(8) :: roh, roe, vsq, bsq, esq, gfl, denr, eem
!
!=====================================================================

  do k=ks1+nm0-1,ke1-nm0
    do j=js1+nm0-1,je1-nm0
      do i=is1+nm0-1,ie1-nm0

        de=uri(1,i,j,k)
        v1=uri(2,i,j,k)
        v2=uri(3,i,j,k)
        v3=uri(4,i,j,k)
        pr=uri(5,i,j,k)
        b1=uri(7,i,j,k)
        b2=uri(8,i,j,k)
        b3=uri(9,i,j,k)
        e1=uri(10,i,j,k)
        e2=uri(11,i,j,k)
        e3=uri(12,i,j,k)
        ch=uri(13,i,j,k)
        ph=uri(14,i,j,k)
        ps=uri(15,i,j,k)
         
        if(ieos .eq. 0) then
          roh=de+(gam/(gam-1.0))*pr
        elseif(ieos .eq. 1) then
          roh=(5./2.)*pr+sqrt((9./4.)*pr**2+de**2)
        elseif(ieos .eq. 2) then
          roe=(3./2.)*(pr+((3.*pr**2)/(2.0*de+sqrt(2.*pr**2+4.*de**2)) ))
          roh=de+roe+pr
        elseif(ieos .eq. 3) then
          roh=(12.*pr**2+8.*pr*de+2.*de**2)/(3.*pr+2.*de)
        endif
          
        vsq=v1*v1+v2*v2+v3*v3
        bsq=(b1*b1+b2*b2+b3*b3)
        esq=(e1*e1+e2*e2+e3*e3)        
        gfl=1.0/sqrt(1.0-vsq)
!
        denr=(gfl**2)*roh
        eem=0.5*(bsq+esq)
          
        uu(1,i,j,k)=gfl*de
        uu(2,i,j,k)=denr*v1+(e2*b3-b2*e3)
        uu(3,i,j,k)=denr*v2+(e3*b1-b3*e1)
        uu(4,i,j,k)=denr*v3+(e1*b2-b1*e2)
        uu(5,i,j,k)=denr-pr+eem
           
        if( iflag(6).le.1 ) then
          uu(6,i,j,k)=gfl*de*x1(i)
        elseif( iflag(6).le.3 ) then
          uu(6,i,j,k)=gfl*pr/de**(gam-1.0)
        elseif( iflag(6).eq.4 ) then
          uu(6,i,j,k)=gfl*de*log(pr/de**gam)
        endif         
           
        uu(7,i,j,k)=b1
        uu(8,i,j,k)=b2
        uu(9,i,j,k)=b3
        uu(10,i,j,k)=e1
        uu(11,i,j,k)=e2
        uu(12,i,j,k)=e3
        uu(13,i,j,k)=ch
!
        uu(14,i,j,k)=ph
        uu(15,i,j,k)=ps
!
      enddo
    enddo
  enddo
      
  return
end subroutine caluu3

